module.exports = function (runtime, global) {
    let OCR = com.stardust.autojs.runtime.api.OCR;

    function $OCR(javaObject) {
        this.javaObject = javaObject;
        $events.on('exit', () => {
            this.release();
        });
    }

    $OCR.create = function (options) {
        options = options || {};
        let parallelThreads = typeof (options.parallelThreads) === 'undefined' ? 4 : options.parallelThreads;
        let cpuPowerMode = typeof (options.cpuPowerMode) === 'undefined' ? "LITE_POWER_HIGH" : options.cpuPowerMode;
        let useOpenCL = typeof (options.useOpenCL) === 'undefined' ? false : options.useOpenCL;
        let models = options.models || 'default';
        let labelsFile = options.labelsFile || null;
        return new $OCR(OCR.create(models, labelsFile, cpuPowerMode, parallelThreads, useOpenCL));
    }

    $OCR.prototype.detect = function (image, options) {
        options = options || {};
        let max = typeof (options.max) === 'undefined' ? 1000 : options.max;
        let detectRotation = typeof (options.detectRotation) === 'undefined' ? false : options.detectRotation;
        return toJsOCRResult(this.javaObject.detect(image, max, detectRotation));
    }

    $OCR.prototype.release = function () {
        this.javaObject.release();
    }

    function OCRResult(javaObject, params) {
        params = params || javaObject;
        this.javaObject = javaObject;
        this.confidence = params.confidence;
        this.text = params.text;
        this.bounds = params.bounds;
        this.rotation = params.rotation < 1e-6 ? 0 : params.rotation;
        this.rotationConfidence = params.rotationConfidence < 1e-6 ? 0 : params.rotationConfidence;
    }

    OCRResult.prototype.clickCenter = function() {
        click(this.bounds.centerX(), this.bounds.centerY());
    }

    $OCR.OCRResult = OCRResult;

    function toJsOCRResult(result) {
        let size = result.size();
        let array = [];
        for (let i = 0; i < size; i++) {
            let r = result.get(i);
            array.push(new OCRResult(r, {
                confidence: r.confidence,
                text: r.words,
                bounds: r.bounds,
                rotation: r.rotation,
                rotationConfidence: r.rotationConfidence,
            }));
        }
        return array;
    }

    return $OCR;
}
